require File.join(File.expand_path('../../../helpers', __FILE__),  "document_spec_helper")

class MyException < Exception

  def initialize(message, foo, bar)
    super(message)
    @foo, @bar = foo, bar
  end

  attr_accessor :foo
  attr_accessor :bar
end

describe "Exception Serialization" do

  before :each do
    @e = MyException.new("Error","foox", 42)
  end

  it "should serialize" do

    r = @e.to_tomte
    expect(r).to be_a_kind_of Hash
    expect(r[:_type]).to eq "MyException"
    expect(r[:message]).to eq "Error"
    expect(r[:attributes][:foo]).to eq "foox"
    expect(r[:backtrace]).to be_a_kind_of Array
  end

  it "should serialize and back" do
    r = @e.to_tomte

    t = Tomte::Document::Parser.load(r)
    expect(t).to be_a_kind_of MyException
    expect(t.message).to eq "Error"
    expect(t.foo).to eq "foox"
    expect(t.bar).to eq 42
  end

  it "should serialize and back (to a generic exception if can not constantize)" do
    r = @e.to_tomte
    r[:_type] = "MissingException"

    t = Exception.from_tomte(r)
    expect(t).to be_a_kind_of GenericError
    expect(t.type).to eq "MissingException"
    expect(t.message).to eq "Error"
    expect(t.attributes.foo).to eq "foox"
    expect(t.attributes.bar).to eq 42
  end

end
